/*
Alolan Geodude Follower Code (Paradise Variant) V2.0
	by Bartman3010
	Based on Follower Code Originally Written by SteelT
	Special thanks to Togen (4/3)
*/

local SKIN = "brock"

freeslot("MT_AGD_FOLL", 
		"SPR_AGDF", 
		"S_AGD_NORM", 
		"S_AGD_DRFTA", 
		"S_AGD_DRFTB", 
		"S_AGD_PAIN",
		"S_AGD_SQSH",
		"S_AGD_WIN1",
		"S_AGD_WIN2",
		"S_AGD_WIN3",
		"S_AGD_WIN4",
		"S_AGD_WIN5",
		"S_AGD_WIN6"
		)

states[S_AGD_NORM] = {
	sprite = SPR_AGDF,
	frame = A,
	nextstate = S_AGD_NORM
}

states[S_AGD_DRFTA] = {
	sprite = SPR_AGDF,
	frame = B,
	nextstate = S_AGD_DRFTA
}

states[S_AGD_DRFTB] = {
	sprite = SPR_AGDF,
	frame = C,
	nextstate = S_AGD_DRFTB
}

states[S_AGD_PAIN] = {
	sprite = SPR_AGDF,
	frame = D,
	nextstate = S_AGD_PAIN
}

states[S_AGD_SQSH] = {
	sprite = SPR_AGDF,
	frame = J,
	nextstate = S_AGD_SQSH
}

states[S_AGD_WIN1] = {
	sprite = SPR_AGDF,
	frame = E,
	tics = 10,
	nextstate = S_AGD_WIN2
}

states[S_AGD_WIN2] = {
	sprite = SPR_AGDF,
	frame = F,
	tics = 3,
	nextstate = S_AGD_WIN3
}

states[S_AGD_WIN3] = {
	sprite = SPR_AGDF,
	frame = G,
	tics = 4,
	nextstate = S_AGD_WIN4
}

states[S_AGD_WIN4] = {
	sprite = SPR_AGDF,
	frame = H,
	tics = 10,
	nextstate = S_AGD_WIN5
}

states[S_AGD_WIN5] = {
	sprite = SPR_AGDF,
	frame = I,
	tics = 3,
	nextstate = S_AGD_WIN6
}

states[S_AGD_WIN6] = {
	sprite = SPR_AGDF,
	frame = G,
	tics = 4,
	nextstate = S_AGD_WIN1
}

mobjinfo[MT_AGD_FOLL] = {
	doomednum = -1,
	spawnstate = S_AGD_NORM,
	spawnhealth = 1,
	painchance = 200,
	speed = 3,
	radius = 16*FRACUNIT,
	height = 32*FRACUNIT,
	dispoffset = 0,
	mass = 100,
	damage = 0,
	reactiontime = 32,
	activesound = sfx_None,
	flags = MF_NOGRAVITY|MF_SCENERY|MF_NOCLIPHEIGHT|MF_DONTENCOREMAP,
	raisestate = S_NULL
}



addHook("MobjThinker", function(mo)
	if mo and mo.skin == "brock_g_p" and mo.player //Skin check
		local player = mo.player
		if not player.mo.a_geodude
			player.mo.a_geodude = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_AGD_FOLL)
			player.mo.a_geodude.angle = player.mo.angle
			player.mo.a_geodude.flags2 = player.mo.flags2
			player.mo.a_geodude.destscale = player.mo.destscale
			player.mo.a_geodude.target = player.mo
		end
	elseif mo and mo.skin ~= "brock_g_p" and mo.a_geodude and mo.a_geodude.valid then //Not using Brock but still has Geodude with them.
		P_RemoveMobj(mo.a_geodude)
		mo.a_geodude = nil
	end
end, MT_PLAYER)

addHook("MobjThinker", function(mo)
	local brock_player = mo.target
	local totz
	local player_drift -- Determine what direction player is drifting in
	local newx
	local newy 
		
	-- Thinker for Geodude
	if mo.target and mo.target.valid and brock_player.player and brock_player.player.valid then
		--Match player colors and effects
		mo.color = brock_player.color
		mo.colorized = brock_player.colorized
		
		local mangle = FixedAngle(((270)) << FRACBITS) --Set Angle Position of Geodude
		player_drift = brock_player.player.kartstuff[k_drift] * ANG1 * 8 -- Set the player drift variable based on player activation.
		-- Get height of player and set Geodude height based on gravity
		if (mo.eflags & MFE_VERTICALFLIP)
			if (brock_player.player.kartstuff[k_squishedtimer] == 0)
				totz = (brock_player.z) - (brock_player.height) + (mo.height)-- Flipped gravity
			else
				totz = (brock_player.z)
			end
		else
			if (brock_player.player.kartstuff[k_squishedtimer] == 0)
				totz = (brock_player.z) + (brock_player.height) - (mo.height / 2) --Normal gravity
			else
				totz = (brock_player.z)
			end
		end
		--Flatten Geodude if player is flattened. Set height to match player's Z height.
		
		mo.angle = (brock_player.angle - player_drift) //Rotate Geodude with the player.
		mo.frameangle = brock_player.frameangle //Geodude's art faces same direction as the player.
		--Match playher flags
		mo.eflags = brock_player.eflags 
		mo.flags2 = brock_player.flags2
		--Match player destscale (grow, shrink etc.)
		mo.destscale = brock_player.destscale
		
		//Sets position of geodude along radius. Sets length of radius.
		newx = sin(mangle - mo.angle) * ((brock_player.radius * 3>>FRACBITS) ) 
		newy = cos(mangle - mo.angle) * ((brock_player.radius * 3>>FRACBITS) )
		//Translates Geodude's position
		P_MoveOrigin(mo, newx + mo.target.x, newy + mo.target.y, totz)
	
		
		-- Change animation states
		if brock_player.player.exiting == 0 then
			if brock_player.player.kartstuff[k_spinouttimer] == 0 and brock_player.health > 0 
				if player_drift == 0
					if not (mo.state == S_AGD_NORM)
						mo.state = S_AGD_NORM
					end
				elseif player_drift < 0
					if not (mo.state == S_AGD_DRFTA)
						mo.state = S_AGD_DRFTA
					end
				elseif player_drift > 0
					if not (mo.state == S_AGD_DRFTB)
						mo.state = S_AGD_DRFTB
					end
				end
			end
			if (brock_player.player.kartstuff[k_spinouttimer] > 0 or brock_player.health == 0 or brock_player.player.kartstuff[k_squishedtimer] > 0) then
				if not (mo.state == S_AGD_PAIN) and (brock_player.player.kartstuff[k_spinouttimer] > 0 or brock_player.health == 0)
					mo.state = S_AGD_PAIN
				end
				if not (mo.state == S_AGD_SQSH) and brock_player.player.kartstuff[k_squishedtimer] > 0
					mo.state = S_AGD_SQSH
				end
				
			end
		end
				--Player won the race (Top bracket)
		if brock_player.player.exiting > 0 and K_IsPlayerLosing(brock_player.player) == false
			if not (mo.state >= S_AGD_WIN1 and mo.state <= S_AGD_WIN6)
				mo.state = S_AGD_WIN1
			end
		elseif brock_player.player.exiting > 0 and K_IsPlayerLosing(brock_player.player) == true
			if not (mo.state == S_AGD_NORM)
				mo.state = S_AGD_NORM
			end
		end

	end	
	
	--Remove Geodude object on respawn
	if mo and (not mo.target or not mo.target.valid) or (not mo.target.player or not mo.target.valid) then
		P_RemoveMobj(mo)
    end
	
end, MT_AGD_FOLL)